@@ -206,7 +206,6 @@ A package including binaries from grub2 and loadlin would have this field in its
           'udeb'
         ],
         'description' => 'If this field is present, the package is not a regular Debian package, but either a udeb generated for the Debian installer or a tdeb containing translated debconf strings.',
-        'experience' => 'advanced',
         'migrate_from' => {
           'formula' => '$xc',
           'variables' => {
@@ -224,7 +223,6 @@ A package including binaries from grub2 and loadlin would have this field in its
           'udeb'
         ],
         'description' => 'If this field is present, the package is not a regular Debian package, but either a udeb generated for the Debian installer or a tdeb containing translated debconf strings.',
-        'experience' => 'advanced',
         'status' => 'deprecated',
         'summary' => 'The type of the package, if not a regular Debian one',
         'type' => 'leaf',
@@ -281,7 +279,6 @@ else {
       },
       'XB-Python-Version',
       {
-        'experience' => 'advanced',
         'status' => 'deprecated',
         'type' => 'leaf',
         'value_type' => 'uniline'
@@ -304,6 +301,13 @@ supported by the library',
             }
           ]
         }
+      },
+      'Build-Profiles',
+      {
+        'description' => 'A space-separated list of optionally negated profile names',
+        'match' => '[!]?[a-z][a-z0-9-]*(\\s+[!]?[a-z][a-z0-9-]*)*',
+        'type' => 'leaf',
+        'value_type' => 'uniline'
       }
     ],
     'license' => 'LGPL2',
