@@ -79,6 +79,26 @@ area/section if the package is in the contrib or non-free archive areas.
       'XS-Testsuite',
       {
         'description' => 'Enable a testsuite to be used with this package. Currently only the \'autopkgtest\' name is allowed. For more details see L<README.package-tests|http://anonscm.debian.org/gitweb/?p=autopkgtest/autopkgtest.git;f=doc/README.package-tests;hb=HEAD>',
+        'status' => 'deprecated',
+        'summary' => 'name of the non regression test suite',
+        'type' => 'leaf',
+        'value_type' => 'uniline',
+        'warn_unless_match' => {
+          '^autopkgtest$' => {
+            'fix' => '$_ = undef; # restore default value',
+            'msg' => 'Currently, only "autopkgtest" is supported'
+          }
+        }
+      },
+      'Testsuite',
+      {
+        'description' => 'Enable a testsuite to be used with this package. Currently only the \'autopkgtest\' name is allowed. For more details see L<README.package-tests|http://anonscm.debian.org/gitweb/?p=autopkgtest/autopkgtest.git;f=doc/README.package-tests;hb=HEAD>',
+        'migrate_from' => {
+          'formula' => '$xs_testsuite',
+          'variables' => {
+            'xs_testsuite' => '- XS-Testsuite'
+          }
+        },
         'summary' => 'name of the non regression test suite',
         'type' => 'leaf',
         'value_type' => 'uniline',
@@ -187,7 +207,7 @@ in Debian developer reference.',
       {
         'compute' => {
           'allow_override' => '1',
-          'formula' => '$maintainer =~ /pkg-perl/ ? "http://anonscm.debian.org/gitweb/?p=pkg-perl/packages/$pkgname.git" : undef ;',
+          'formula' => '$maintainer =~ /(pkg-(?:perl|ruby-extras))/ ? "http://anonscm.debian.org/gitweb/?p=$1/packages/$pkgname.git" : undef ;',
           'use_eval' => '1',
           'variables' => {
             'maintainer' => '- Maintainer',
@@ -274,7 +294,7 @@ The information is meant to be useful for a user knowledgeable in the given Vers
       {
         'compute' => {
           'allow_override' => '1',
-          'formula' => '$maintainer =~ /pkg-perl/ ? "git://anonscm.debian.org/pkg-perl/packages/$pkgname.git" : \'\' ;',
+          'formula' => '$maintainer =~ /(pkg-(?:perl|ruby-extras))/ ? "git://anonscm.debian.org/$1/packages/$pkgname.git" : \'\' ;',
           'use_eval' => '1',
           'variables' => {
             'maintainer' => '- Maintainer',
@@ -358,7 +378,6 @@ The information is meant to be useful for a user knowledgeable in the given Vers
       },
       'XS-Python-Version',
       {
-        'experience' => 'advanced',
         'status' => 'deprecated',
         'type' => 'leaf',
         'value_type' => 'uniline'
@@ -366,7 +385,6 @@ The information is meant to be useful for a user knowledgeable in the given Vers
       'X-Python-Version',
       {
         'description' => 'This field specifies the versions of Python (not versions of Python 3) supported by the source package.  When not specified, they default to all currently supported Python (or Python 3) versions. For more detail, See L<python policy|http://www.debian.org/doc/packaging-manuals/python-policy/ch-module_packages.html#s-specifying_versions>',
-        'experience' => 'advanced',
         'migrate_from' => {
           'formula' => 'my $old = $xspython ;
 my $new ;
@@ -397,7 +415,6 @@ $new ;',
       'X-Python3-Version',
       {
         'description' => 'This field specifies the versions of Python 3 supported by the package. For more detail, See L<python policy|http://www.debian.org/doc/packaging-manuals/python-policy/ch-module_packages.html#s-specifying_versions>',
-        'experience' => 'advanced',
         'summary' => 'supported versions of Python3 ',
         'type' => 'leaf',
         'value_type' => 'uniline'
