@@ -118,12 +118,19 @@ To upgrade your file from an old spec, run:
           'config_class_name' => 'Dpkg::Copyright::Content',
           'type' => 'node'
         },
-        'description' => 'Patterns indicating files having the same license and sharing copyright holders. See "File patterns" below',
+        'description' => 'Patterns indicating files having the same license and sharing copyright holders.
+See L<files pattern documentation|https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/#files-field> for details.',
         'index_type' => 'string',
         'ordered' => '1',
         'type' => 'hash',
         'warn_if_key_match' => '[\\[\\]\\|]'
       },
+      'Files-Excluded',
+      {
+        'description' => 'White space separated list of file patterns to exclude from the package. This field is only used by L<uscan>. Example: C<*/Makefile.in aclocal.m4>. See also L<UscanEnhancements|https://wiki.debian.org/UscanEnhancements> and the L<files pattern documentation|https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/#files-field>.',
+        'type' => 'leaf',
+        'value_type' => 'uniline'
+      },
       'Global-License',
       {
         'config_class_name' => 'Dpkg::Copyright::GlobalLicense',
@@ -180,8 +187,7 @@ To upgrade your file from an old spec, run:
           'type' => 'node'
         },
         'index_type' => 'string',
-        'type' => 'hash',
-        'warn_if_key_match' => '\\s'
+        'type' => 'hash'
       },
       'X-Comment',
       {
