@@ -165,8 +165,10 @@ providing the following file:
     {
         # test for XS-Testsuite field
         name => 'xs-testsuite',
+        load_warnings => [ qr/deprecated/  ],
+        apply_fix => 1,
         check => {
-            'source XS-Testsuite' => 'autopkgtest',
+            'source Testsuite' => 'autopkgtest',
         },
     },
     {
@@ -177,6 +179,14 @@ providing the following file:
             'source Section' => 'gnu-r',
             'binary:gnu-r-view Section' => 'gnu-r',
         ]
+    },
+    {
+        name => 'build-profiles',
+        load_warnings => [ qr/standards version/ ],
+        check => {
+            'binary:pkg-config Build-Profiles' => '!stage1',
+            'binary:pkg-config-stage1 Build-Profiles' => 'stage1',
+        },
     }
 );
 
